!==========================================================*
!                                                          *
!     2D poststack depth Migration with Multi_reference    *
!     veloxity and Split_step Fourier propagator           *
!==========================================================*
!     DEPARTMENT OF MARINE GEOLOGY AND GEOPHYSICS          *
!     TONGJI UNIVERSITY                                    *
!==========================================================*
!     REMARKS:                                             *
!     AUTHOR :   Herb.Wang                                 *
!     DATE   :   1999.6-7                                  *
!     Modified by Herb.Wang                                *
!     DATE   :   2001.7-8                                  *
!     Modified by Herb.Wang                                *
!     DATE   :   2005.4                                    * 
!==========================================================*
PROGRAM MULTIPLE_reference_velocity_POSTSTACK_2D_DPMIG
use sep3d_struct_mod
use sep  
use CONTROL_SUBROUTINE
    
implicit none

type(sep3d):: Velocity, InputData, Imageout, Refvout
Real       :: f1, f2, f3, f4
Real       :: DIP_max, Threshold  
                                                                                                            
integer :: velnz, velnx, datant, datanx
real    :: veldz, veldx, veloz, velox, datadt, datadx, dataot, dataox
real    :: Depth,xleft_max,xright_min
integer :: NX,Nz_step

call initpar()
                                                 
call from_param("f1", f1, 2.0)
call from_param("f2", f2, 8.0)
call from_param("f3", f3, 42.0)
call from_param("f4", f4, 50.0)
call from_param("DIPmax", DIP_max, 75.0)
call from_param("Threshold", Threshold, 1.5)
 
call init_sep3d("Velocity", velocity, "INPUT")
velnz=velocity%n(1); velnx=velocity%n(2)
veldz=velocity%d(1); veldx=velocity%d(2)
veloz=velocity%o(1); velox=velocity%o(2)
call init_sep3d("Inputdata", Inputdata, "INPUT")
datant=Inputdata%n(1); datanx=Inputdata%n(2)
datadt=Inputdata%d(1); datadx=Inputdata%d(2)
dataot=Inputdata%o(1); dataox=Inputdata%o(2)

call from_param("Depth", Depth, veldz*velnz)

if(datadx .ne. veldx) then
 write(*,*) 'velocity field interval is not equal to stack profile interval'
 stop
endif
if(Depth .gt. veldz*velnz) then
 write(*,*) 'the defined extrapolation depth is too large'
 stop
endif

xleft_max=max(dataox, velox)
xright_min=min(velox+veldx*(velnx-1), dataox+datadx*(datanx-1))

NX=(xright_min-xleft_max)/veldx+1

!write(*,*)"aaaaaaaaaaaaaaaaaaaaaaaa"
call init_sep3d(Imageout, "OUTPUT", "FLOAT", "REGULAR", ctag="Image")
call axis_allocate(Imageout, 2)

Nz_step=Depth/veldz

!write(*,*)"bbbbbbbbbbbbbbbbbbbbbbbbb"
!write(*,*)"Nz_step,veldz,veloz,NX,veldx,xleft_max="
!write(*,*) Nz_step,veldz,veloz,NX,veldx,xleft_max
!write(*,*)"ddddddddddddddddddddddd"

Imageout%n(1)=Nz_step ; Imageout%n(2)=NX 
Imageout%d(1)=veldz   ; Imageout%d(2)=veldx 
Imageout%o(1)=veloz   ; Imageout%o(2)=xleft_max 

call sep3d_set_sep3d(Imageout)
call sep3d_write_description("Image", Imageout)

call init_sep3d(Refvout, "OUTPUT", "FLOAT", "REGULAR", ctag="Refveo")
call axis_allocate(Refvout, 2)
Refvout%n(1)=Nz_step ; Refvout%n(2)=NX
Refvout%d(1)=veldz   ; Refvout%d(2)=veldx
Refvout%o(1)=veloz   ; Refvout%o(2)=xleft_max
call sep3d_set_sep3d(Refvout)
call sep3d_write_description("Refveo", Refvout)

CALL POSTSTACK_2D_DPMIG(velnz, velnx, datant, datanx, veldz, veldx, &
     veloz, velox, datadt, datadx, dataot, dataox, NX, Nz_step, &
     f1, f2, f3, f4, DIP_max, Threshold)

WRITE(*,*)'*+++++++++++++++++++++++++++++++++++++*'
WRITE(*,*)'*  MIGRATION PROCESSING IS FINISHED   *'
WRITE(*,*)'*+++++++++++++++++++++++++++++++++++++*'
 
END program MULTIPLE_reference_velocity_POSTSTACK_2D_DPMIG

